<?php
/**
 * Prestashop module for Oct8ne
 *
 * @category  Prestashop
 * @category  Module
 * @author    Prestaquality.com
 * @copyright 2016 Prestaquality
 * @license   Commercial license see license.txt
 * Support by mail  : info@prestaquality.com
 *
 * Modificado por PresTeamShop.com - 2019
 * Support: info@presteamshop.com
 */

class Oct8neOct8neConnectorModuleFrontController extends ModuleFrontController
{
    /*public $ssl = true;*/
    private $callback;
    private $octmethod;

    private $address;

    private $name_controller = 'oct8neconnector';

    private $include_subcategories = true;

    /**
     * Inicializacion
     */
    public function init()
    {
        parent::init();

        $this->callback = trim(strip_tags(Tools::getValue('callback')));
        $this->octmethod = trim(strip_tags(Tools::getValue('octmethod')));

        $this->address = null;
    }

    /**
     * Metodo que se ejecuta despues de la inicializacion. Comprueba si existe el metodo solicitado
     * y lo ejecuta si es el caso.
     */
    public function postProcess()
    {

        $data = "";
        try {

            $meth = $this->octmethod . 'OctMeth';

            if (method_exists($this, $meth)) {
                $data = $this->$meth();

            } else {

                throw new Exception("Method not Exists");
            }

        } catch (Exception $ex) {
            if (Tools::getIsset('debug')) {
                die($ex);
            } else {
                $this->module->logException($ex);
            }

        }
        $this->postProcessFinally($data);

    }

    /**
     * Acciones finales
     * @param $data
     */
    private function postProcessFinally($data)
    {

        ob_end_clean();
        $this->printJsonCode($data);
        die();
    }

    /**
     * Metodo que establece la moneda local
     */
    private function setContextCurrency()
    {

        $currency = trim(strip_tags(Tools::getValue('currency')));

        if (!empty($currency)) {

            $in_currency = Currency::getIdByIsoCode($currency);

            if (!empty($in_currency)) {

                //Cargamos el objeto, comprobamos que lo tenemos y que es válido
                $currency = new Currency($in_currency);

                if (Validate::isLoadedObject($currency) && $currency->active == '1') {
                    Context::getContext()->currency = $currency;
                }
            }
        }
    }

    /**
     * Metodo que establece la lengua local
     */
    private function setContextLanguage()
    {
        $language = trim(strip_tags(Tools::getValue('locale')));


        if (!empty($language)) {

            $global_local = explode('-', $language);

            $id_language = Language::getIdByIso($global_local[0]);

            if (!empty($id_language)) {

                //Cargamos el objeto, comprobamos que lo tenemos y que es valido
                $language = new Language($id_language);

                if (Validate::isLoadedObject($language) && $language->active == '1') {
                    Context::getContext()->language = $language;
                }
            }
        }

    }

    /**
     * Metodo que inicia sesion del clientId enviado
     */
    private function setContextCustomer()
    {
//        $origin = trim(strip_tags(Tools::getValue('origin')));
//
//        $email_customer = trim(strip_tags(Tools::getValue('clientId')));
//        if (!empty($email_customer) && Validate::isEmail($email_customer)) {
//            $id_customer = Customer::customerExists($email_customer, true);
//            if ($id_customer) {
//                $customer = new Customer((int)$id_customer);
//                if (Validate::isLoadedObject($customer)) {
//                    Context::getContext()->cookie->id_customer        = (int) $customer->id;
//                    Context::getContext()->cookie->customer_lastname  = $customer->lastname;
//                    Context::getContext()->cookie->customer_firstname = $customer->firstname;
//                    Context::getContext()->cookie->logged             = 1;
//                    $customer->logged                                 = 1;
//                    Context::getContext()->cookie->is_guest           = $customer->isGuest();
//                    Context::getContext()->cookie->passwd             = $customer->passwd;
//                    Context::getContext()->cookie->email              = $customer->email;
//
//                    // Add customer to the context
//                    Context::getContext()->customer = $customer;
//                }
//            }
//        } else {
//            if ($origin == 'agent' && Validate::isLoadedObject(Context::getContext()->customer)) {
//                Context::getContext()->customer->logout();
//                Context::getContext()->customer = new Customer();
//            }
//        }
    }
    
    private function logoutContextCustomer(){
//        $origin = trim(strip_tags(Tools::getValue('origin')));
//        
//        if ($origin == 'agent' && Validate::isLoadedObject(Context::getContext()->customer)) {
//            Context::getContext()->customer->logout();
//            Context::getContext()->customer = new Customer();
//        }  
    }

    /**
     * Metodo que estable el id_country del pais se conectan
     */
    private function setContextCountry()
    {
        $this->address = Address::initialize();

        $iso_code_country = trim(strip_tags(Tools::getValue('country')));
        if (!empty($iso_code_country) && Validate::isGenericName($iso_code_country)) {
            $id_country = Country::getByIso($iso_code_country);
            if ($id_country) {
                $this->address->id_country = (int)$id_country;
            }
        }
    }

    private function getPrice($id_product, $usetax = true, $id_product_attribute = null, $usereduc = true)
    {
        $id_customer = null;
        $id_group = null;
        $specific_price_output = null;

        if (Validate::isLoadedObject(Context::getContext()->customer)) {
            $id_customer = (int) Context::getContext()->customer->id;
            $id_group = (int)Customer::getDefaultGroupId($id_customer);
        } else {
            $id_group = (int)Group::getCurrent()->id;
        }

        $id_country = (int) $this->address->id_country;
        $id_state = (int) $this->address->id_state;
        $zipcode = $this->address->postcode;

        $return = Product::priceCalculation(
            Context::getContext()->shop->id,
            $id_product,
            $id_product_attribute,
            $id_country,
            $id_state,
            $zipcode,
            Context::getContext()->currency->id,
            $id_group,
            1,//quantity
            $usetax,
            6,//decimals
            false,//only_reduc
            $usereduc,//usereduc
            true,//with_ecotax
            $specific_price_output,//specific_price_output
            true,//use_group_reduction
            $id_customer,
            true,//use_customer_price
            null,//id_cart
            0,//cart_quantity
            null//id_customization
        );

        return $return;
    }

    /**
     * @param $data
     * Imprime un json de respuesta
     */
    public function printJsonCode($data)
    {

        if (empty($this->callback)) {
            header('Content-Type: application/json; charset=UTF-8');
            
            if (version_compare(_PS_VERSION_, '8.0', '<')) {
                $json = Tools::jsonEncode($data);
            }else{
                $json = json_encode($data);
            }
            
            echo $json;

        } else {
            $result ="";
            if (version_compare(_PS_VERSION_, '8.0', '<')) {
                $result = Tools::jsonEncode($data);
            }else{
                $result = json_encode($data);
            }
            
            header('Content-Type: application/javascript; charset=UTF-8');
            $json = $this->callback . '(' . $result . ');';
            echo $json;
        }
    }

    /**
     * Obtiene unos datos concretos sobre un producto, a modo de resumen simple
     * @param $id_product
     * @return array
     */
    public function getProductSummary($id_product)
    {
        $product = new Product($id_product, false, Context::getContext()->language->id);

        $result = new stdClass();
        if (Validate::isLoadedObject($product)) {
            if ($product->active) {
                $image = Product::getCover($id_product);
                $image = $image['id_image'];
                
                if (!isset($image)) {
                    $allImages = $product->getImages(Context::getContext()->language->id);
                    foreach ($allImages as $firstImage) {
                        $image = $firstImage['id_image'];
                        break;
                    }
                }

                if (isset($image)) {
                    $thumbnail = Context::getContext()->link->getImageLink($product->link_rewrite, $image);
                    $url_image_type = Configuration::get('OCT_URL_IMG_TYPE');

                    if ($url_image_type == 2) {
                        $base_url = rtrim($this->context->shop->getBaseURL(), '/');
                        $base_url = str_replace("http://", "", $base_url);
                        $base_url = str_replace("https://", "", $base_url);
                        $base_url = $base_url . "/";

                        $thumbnail_exploded = explode($base_url, $thumbnail);
                        $thumbnail = $thumbnail_exploded[0] . $base_url . $id_product . "-" . $thumbnail_exploded[1];
                    }
                } else {
                    $thumbnail = "";                    
                }

                $allow_out_stock = true;
                if ($product->out_of_stock == 0) {
                    $allow_out_stock = false;
                } else if ($product->out_of_stock == 2) {
                    $allow_out_stock = (bool) Configuration::get('PS_ORDER_OUT_OF_STOCK');
                }

                $result = array(
                    "internalId" => $product->id,
                    "title" => $product->name,
                    "formattedPrice" => Tools::displayPrice($this->getPrice($product->id, Product::$_taxCalculationMethod == PS_TAX_INC)),
                    "formattedPrevPrice" => Tools::displayPrice($this->getPrice($product->id, Product::$_taxCalculationMethod == PS_TAX_INC, null, false)),
                    "productUrl" => Context::getContext()->link->getProductLink($id_product),
                    "thumbnail" => $thumbnail,
                    'configurable' => false,
                    'stock' => array(
                        'quantity' => StockAvailable::getQuantityAvailableByProduct($product->id),
                        'minimal_quantity' => (int)$product->minimal_quantity,
                        'allowOutStock' => $allow_out_stock,
                        'labelInStock' => $product->available_now,
                        'labelOutStock' => $product->available_later,
                    )
                );

                $sqlAttributes = new DbQuery();
                $sqlAttributes->from('product_attribute', 'pa');
                $sqlAttributes->innerJoin('product_attribute_shop', 'pas', 'pa.id_product_attribute = pas.id_product_attribute');
                $sqlAttributes->where('pa.id_product = '.(int)$product->id);
                $sqlAttributes->where('pas.id_shop = '.(int)$this->context->shop->id);
                $sqlAttributes->orderBy('pa.`id_product_attribute`');
                $resultAttributes = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sqlAttributes);

                if (Tools::strtolower($this->octmethod) == 'productinfo') {
                    if ($resultAttributes) {
                        $variations = array();
                        $attributes_group = array();

                        foreach($resultAttributes as $i => $rowA) {
                            $id_product_attribute = (int)$rowA['id_product_attribute'];
                            $attributes = array();

                            $sqlAttributeGroup = new DbQuery();
                            $sqlAttributeGroup->select('agl.public_name, al.name');
                            $sqlAttributeGroup->from('product_attribute_combination', 'pac');
                            $sqlAttributeGroup->innerJoin('attribute', 'a', 'a.id_attribute = pac.id_attribute');
                            $sqlAttributeGroup->innerJoin('attribute_lang', 'al', 'al.id_attribute = a.id_attribute');
                            $sqlAttributeGroup->innerJoin('attribute_group', 'ag', 'ag.id_attribute_group = a.id_attribute_group');
                            $sqlAttributeGroup->innerJoin('attribute_group_lang', 'agl', 'agl.id_attribute_group = a.id_attribute_group');
                            $sqlAttributeGroup->where('pac.id_product_attribute = '.$id_product_attribute);
                            $sqlAttributeGroup->where('al.id_lang = '.(int)Context::getContext()->language->id);
                            $sqlAttributeGroup->where('agl.id_lang = '.(int)Context::getContext()->language->id);
                            $resultAttributeGroup = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sqlAttributeGroup);

                            if ($resultAttributeGroup) {
                                foreach ($resultAttributeGroup as $rowG) {
                                    if ($i == 0) {
                                        $attributes_group[] = $rowG['public_name'];
                                    }

                                    $attributes[] = array(
                                        $rowG['public_name'] => $rowG['name']
                                    );
                                }

                                $result['attributes'] = $attributes_group;
                                $result['variations'][] = array(
                                    'id' => $id_product_attribute,
                                    'attributes' => $attributes,
                                    'formattedPrice' => Tools::displayPrice($this->getPrice($product->id, Product::$_taxCalculationMethod == PS_TAX_INC, $id_product_attribute)),
                                    'formattedPrevPrice' => Tools::displayPrice($this->getPrice($product->id, Product::$_taxCalculationMethod == PS_TAX_INC, $id_product_attribute, false)),
                                    'quantity' => (int) StockAvailable::getQuantityAvailableByProduct($product->id, $id_product_attribute),
                                    'minimal_quantity' => (int)$rowA['minimal_quantity'],
                                    'default' => (bool)$rowA['default_on']
                                );
                            }
                        }

                        unset($result['configurable']);
                    }
                } else {
                    $result['configurable'] = count($resultAttributes) > 0 ? true : false;
                }

                $samePrice = true;

                if (isset($result['variations'])) {
                    $provisionalPrice = null;
                    $variations = $result['variations'];

                    for($i = 0; $i < count($variations); ++$i) {
                        if(isset($provisionalPrice)){
                            if($provisionalPrice != $variations[$i]['formattedPrice']){
                                $samePrice = false;
                            }
                        }else{
                            $provisionalPrice = $variations[$i]['formattedPrice'];
                        }
                    }
                }

                //si tiene ipa los precios se devuelven como null
                $hasipa = ($product->cache_default_attribute > 0);
                if ($hasipa && !$samePrice) {
                    $result['formattedPrice'] = null;
                    $result['formattedPrevPrice'] = null;
                }
            }
        }

        return $result;
    }


    /**
     * Contiene informacion sobre el producto
     * @return array
     */
    public function productInfoOctMeth()
    {
        $this->setContextLanguage();
        $this->setContextCurrency();
        //$this->setContextCustomer();
        $this->setContextCountry();
        $extraProperties = trim(strip_tags(Tools::getValue('extraproperties')));

        $ids = trim(strip_tags(Tools::getValue('productIds')));
        $ids = str_replace(' ', '', $ids);
        $ids = explode(',', $ids);

        $result = array();
        foreach ($ids as $id) {

            $aux = $this->getProductSummary($id);
            if (!empty($aux)) {

                $product = new Product($aux["internalId"], false, Context::getContext()->language->id);
                $aux["description"] = $product->description;
                if(isset($extraProperties) && $extraProperties == "tags"){
                    
                    $tags = Tag::getProductTags($aux["internalId"]);
                    $aux["tags"] = $tags[Context::getContext()->language->id];
                }
                $attr = $product->hasAttributes();
                if ($attr > 0 || !$product->checkQty(1)) {
                    $aux["addToCartUrl"] = "";
                    $aux["useProductUrl"] = true;
                } else {

                    $auxurl = Context::getContext()->link->getPageLink('cart', true, Context::getContext()->language->id, array('add' => 1, 'id_product' => $aux["internalId"], 'token' => Tools::getToken(false), 'qty' => 1));
                    $auxurl = Oct8ne::removeHttProtocol($auxurl);
                    $aux["addToCartUrl"] = $auxurl;
                    $aux["useProductUrl"] = false;
                }

                $images = $product->getImages(Context::getContext()->language->id);

                $medias = array();

                foreach ($images as $image) {

                    $url_image_type = Configuration::get('OCT_URL_IMG_TYPE');

                    $thumbnail = Context::getContext()->link->getImageLink($product->link_rewrite, $image['id_image']);

                    if ($url_image_type == 2) {

                        if (!isset($base_url)) {

                            $base_url = $this->context->shop->getBaseURL();
                            $base_url = str_replace("http://", "", $base_url);
                            $base_url = str_replace("https://", "", $base_url);
                        }

                        $thumbnail_exploded = explode($base_url, $thumbnail);

                        $thumbnail = $thumbnail_exploded[0] . $base_url . $id . "-" . $thumbnail_exploded[1];

                    }

                    $medias[] = array("url" => $thumbnail);
                }

                $aux["medias"] = $medias;

                $result[] = $aux;
            }
        }
        //$this->logoutContextCustomer();

        return $result;
    }

    /**
     * Obtiene informacion simple sobre el producto. LLama a getSummary
     * @return array
     */
    public function productSummaryOctMeth()
    {
        $this->setContextLanguage();
        $this->setContextCurrency();
        //$this->setContextCustomer();
        $this->setContextCountry();

        $ids = trim(strip_tags(Tools::getValue('productIds')));


        $ids = explode(',', $ids);

        $result = array();
        foreach ($ids as $id) {

            $aux = $this->getProductSummary($id);
            if (!empty($aux)) {

                $result[] = $aux;
            }
        }
        //$this->logoutContextCustomer();

        return $result;
    }


    /**
     * Devuelve la busqueda de productos por los paremetros de entrada especificados
     * @return array
     */
    public function searchOctMeth()
    {
        $ids_product = array();
        $id_lang = (int)Context::getContext()->language->id;
        $is_filter_applied = false;

        $this->module->loadLibrary("OctSearchFactory", "search");

        $this->setContextLanguage();
        $this->setContextCurrency();
        //$this->setContextCustomer();
        $this->setContextCountry();

        $result = array();
        $result["total"] = 0;
        $result["results"] = array();
        $result["filters"] = array("applied" => array(), "available" => array());

        //parametro de busqueda
        $search = trim(strip_tags(Tools::getValue('search')));

        //Page por defecto 1
        $page = (int)trim(strip_tags(Tools::getValue('page')));
        if (empty($page)) {
            $page = 1;
        }

        //PageSize por defecto 10
        $pageSize = (int)trim(strip_tags(Tools::getValue('pageSize')));
        if (empty($pageSize)) {
            $pageSize = 10;
        }

        if (!empty($search)) {
            //parametro de ordenacion
            $orderBy = trim(strip_tags(Tools::getValue('orderby')));
            if (!isset($orderBy)) {
                $orderBy = "position";
            } else {

                switch ($orderBy) {

                    case "relevance":
                        $orderBy = "position";
                        break;

                    case "price":
                        break;

                    case "name":
                        break;

                    default:
                        $orderBy = "position";
                        break;
                }
            }

            //ascendente o descendente
            $dir = trim(strip_tags(Tools::getValue('dir')));
            if (empty($dir)) {
                $dir = "asc";
            } else {

                switch ($dir) {

                    case "asc":
                        break;

                    case "desc":
                        break;

                    default:
                        $dir = "asc";
                        break;

                }
            }

            //Factoria de motores de busqueda
            $factory = new OctSearchFactory();

            //Motor de busqueda concreto
            $motor = $factory->getInstance(Configuration::get('OCT_SEARCH_ENGINE'));

            //Realizar la busqueda
            $aux = $motor->doSearch($id_lang, $search, 1, 999, $orderBy, $dir);
            if (isset($aux) && $aux["total"] > 0) {
                $result["total"] = 0;

                foreach ($aux["result"] as $i => $item) {
                    if ($this->context->shop->domain === 'www.decantalo.com') {
                        if (array_key_exists('id_product_attribute', $item) && !empty($item['id_product_attribute'])) {
                            $isAttributeDefault = new DbQuery();
                            $isAttributeDefault->from('product_attribute', 'pa');
                            $isAttributeDefault->where('pa.id_product_attribute = '.(int)$item['id_product_attribute']);
                            $isAttributeDefault->where('pa.default_on = 1');

                            $resultIsAttributeDefault = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($isAttributeDefault);
                            if (!$resultIsAttributeDefault) {
                                continue;
                            }
                        }
                    }

                    $product_ok = $this->isProductAvailableByFilter($item['id_product'], $result, $ids_product, $is_filter_applied, $page, $pageSize);

                    if ($product_ok) {
                        array_push($result["results"], $this->getProductSummary((int)$item['id_product']));
                        array_push($ids_product, (int)$item['id_product']);

                        $result["total"] += 1;
                    }
                }

                $offset = ($page - 1) * $pageSize;
                if ($offset < 0) {
                    $offset = 0;
                }

                $result["results"] = array_slice($result["results"], $offset, $pageSize);
            }
        } else {
            $this->isProductAvailableByFilter(false, $result, $ids_product, $is_filter_applied, $page, $pageSize);
        }

        //Filtros disponibles
        //--------------------------------------------------------------------------------------------------
        if ($ids_product) {
            $this->getFiltersCategory($result, $ids_product);
            $this->getFiltersManufacturer($result, $ids_product);
            $this->getFiltersAttribute($result, $ids_product);
            $this->getFiltersFeature($result, $ids_product);
            $this->getFiltersStock($result, $ids_product);
        }
        //$this->logoutContextCustomer();

        return $result;
    }

    public function isProductAvailableByFilter($id_product, &$result, &$ids_product, &$is_filter_applied, $page, $pageSize)
    {
        $available = true;
        $filter_available = array('category', 'manufacturer', 'stock');
        $total_products = 0;
        $queries = array();

        foreach ($_REQUEST as $key => $filter) {
            $count_filter = 0;
            $filter = trim(strip_tags($filter));
            if (!empty($filter)) {
                $query = new DbQuery();
                $query->select('DISTINCT p.id_product');
                $query->from('product', 'p');
                $query->innerJoin('product_shop', 'ps', 'ps.id_product = p.id_product');
                if ($id_product) {
                    $query->where('p.id_product = '.(int)$id_product);
                }
                $query->where('ps.active = 1');

                $count_filter_aux = count(explode(',', $filter));
                if ($count_filter_aux > $count_filter) {
                    $count_filter = $count_filter_aux;
                }

                if (strpos($key, 'feature_') !== false) {
                    $query->innerJoin('feature_product', 'fp', 'fp.id_product = ps.id_product');
                    $query->innerJoin('feature_value', 'fv', 'fv.id_feature_value = fp.id_feature_value');
                    $query->where('fv.id_feature_value IN ('.$filter.')');
                } else if (strpos($key, 'attribute_') !== false) {
                    $query->innerJoin('product_attribute', 'pa', 'pa.id_product = ps.id_product');
                    $query->innerJoin('product_attribute_combination', 'pac', 'pac.id_product_attribute = pa.id_product_attribute');
                    $query->innerJoin('attribute', 'a', 'a.id_attribute = pac.id_attribute');
                    $query->where('a.id_attribute IN ('.$filter.')');
                } else if (in_array($key, $filter_available)) {
                    if ($key == 'category') {
                        if ($this->include_subcategories) {
                            $categories = array();
                            $categories_sent = explode(',', $filter);

                            foreach ($categories_sent as $id_category) {
                                array_push($categories, $id_category);

                                $this->getIdsCategoriesRecursive($categories, $id_category);
                            }

                            $filter = implode(',', $categories);
                        }

                        $query->innerJoin('category_product', 'cp', 'cp.id_product = ps.id_product');
                        $query->where('cp.id_category IN ('.$filter.')');
                    } else if ($key == 'manufacturer') {
                        $query->where('p.id_manufacturer IN ('.$filter.')');
                    } else if ($key == 'stock') {
                        $query->innerJoin('stock_available', 'sa', 'sa.id_product = ps.id_product');
                        $query->where('sa.id_product_attribute = 0');
                        if ((int)$filter === 1) {
                            $query->where('sa.quantity > 0');
                        } else {
                            $query->where('sa.quantity <= 0');
                        }
                    }
                } else {
                    continue;
                }

                if ($count_filter > 1) {
                    $query->groupBy('p.id_product');
                    $query->having('COUNT(p.id_product) >= '.(int)$count_filter);
                }

                array_push($queries, $query->build());
            }
        }

        if ($queries) {
            $products = array();

            foreach ($queries as $query) {
                $rows = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query);
                $products[] = array_column($rows, 'id_product');
            }

            if (count($products) > 1) {
                $products = call_user_func_array('array_intersect', $products);
            } else {
                $products = $products[0];
            }

            $total_products = count($products);

            if (!$id_product && $products) {
                $ids_product = $products;

                //para aplicar paginacion.
                $query = new DbQuery();
                $query->select('p.id_product');
                $query->from('product', 'p');
                $query->innerJoin('product_shop', 'ps', 'ps.id_product = p.id_product');
                $query->where('p.id_product IN ('.implode(',', $products).') AND ps.id_shop = '.(int)$this->context->shop->id);
                $query->where('ps.active = 1');
                $query->limit($pageSize, (($page - 1) * $pageSize));

                $products = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query);
                $products = array_column($products, 'id_product');
            }

            if ($products && empty($id_product)) {
                foreach ($products as $id_product_tpm) {
                    array_push($result["results"], $this->getProductSummary((int)$id_product_tpm));
                    array_push($ids_product, (int)$id_product_tpm);
                }

                $result["total"] += $total_products;
            } else if (!$products) {
                $available = false;
            }

            $is_filter_applied = true;
        }

        return $available;
    }

    public function getFiltersFeature(&$result, $ids_product = array())
    {
        $id_lang = Context::getContext()->language->id;

        $filters = array();
        foreach ($_REQUEST as $key => $filter) {
            $filter = trim(strip_tags($filter));
            if (!empty($filter)) {
                if (strpos($key, 'feature_') !== false) {
                    if (strpos($filter, ',') === false) {
                        array_push($filters, $filter);
                    } else {
                        $filter = explode(',', $filter);
                        if ($filter) {
                            foreach ($filter as $value) {
                                array_push($filters, $value);
                            }
                        }

                    }
                }
            }
        }

        //Filtros disponibles
        //----------------------------------------------------
        $query = new DbQuery();
        $query->select('fl.name, f.id_feature, fv.id_feature_value, fvl.value, count(*) as count');
        $query->from('feature', 'f');
        $query->innerJoin('feature_lang', 'fl', 'f.id_feature = fl.id_feature AND fl.id_lang = '.(int)$id_lang);
        $query->innerJoin('feature_value', 'fv', 'f.id_feature = fv.id_feature');
        $query->innerJoin('feature_value_lang', 'fvl', 'fv.id_feature_value = fvl.id_feature_value AND fl.id_lang = '.(int)$id_lang);
        $query->innerJoin('feature_product', 'fp', 'fv.id_feature_value = fp.id_feature_value');
        if ($ids_product) {
            $query->where('fp.id_product IN ('.implode(',', $ids_product).')');
        }
        $query->groupBy('f.id_feature, fv.id_feature_value');

        $result_filter = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query);

        if (is_array($result_filter) && count($result_filter)) {
            $filters_availables = array();

            foreach ($result_filter as $row) {
                //Filtros aplicados
                //----------------------------------------------------
                if ($filters) {
                    foreach ($filters as $i => $id_filter) {
                        $filter = new FeatureValue((int)$id_filter, $id_lang);
                        if ($filter->id_feature == $row['id_feature'] && $filter->id == $row['id_feature_value']) {
                            if (Validate::isLoadedObject($filter)) {
                                $result["filters"]['applied'][] = array(
                                    'param' => 'feature_'.Tools::link_rewrite($row['name']),
                                    'paramLabel' => $row['name'],
                                    'value' => (int)$filter->id,
                                    'valueLabel' => $filter->value
                                );

                                unset($filters[$i]);

                                continue 2;
                            }
                        }
                    }
                }
                //----------------------------------------------------

                $filters_availables[Tools::link_rewrite($row['name'])]['id_feature_value'] = $row['name'];
                $filters_availables[Tools::link_rewrite($row['name'])]['name'] = $row['name'];
                $filters_availables[Tools::link_rewrite($row['name'])]['options'][] = array(
                    'value' => (int)$row['id_feature_value'],
                    'valueLabel' => $row['value'],
                    'count' => (int)$row['count']
                );
            }

            if ($filters_availables) {
                foreach ($filters_availables as $key => $value) {
                    $filter_available = array();
                    $filter_available['param'] = 'feature_'.$key;
                    $filter_available['paramLabel'] = $value['name'];
                    $filter_available['options'] = $value['options'];

                    array_push($result["filters"]['available'], $filter_available);
                }
            }
        }
    }

    public function getFiltersAttribute(&$result, $ids_product = array())
    {
        $id_lang = Context::getContext()->language->id;

        $filters = array();
        foreach ($_REQUEST as $key => $filter) {
            $filter = trim(strip_tags($filter));
            if (!empty($filter)) {
                if (strpos($key, 'attribute_') !== false) {
                    if (strpos($filter, ',') === false) {
                        array_push($filters, $filter);
                    } else {
                        $filter = explode(',', $filter);
                        if ($filter) {
                            foreach ($filter as $value) {
                                array_push($filters, $value);
                            }
                        }

                    }
                }
            }
        }

        //Filtros disponibles
        //----------------------------------------------------
        $query = new DbQuery();
        $query->select('agl.public_name, ag.id_attribute_group, al.id_attribute, al.name, count(*) as count');
        $query->from('attribute_group', 'ag');
        $query->innerJoin('attribute_group_lang', 'agl', 'ag.id_attribute_group = agl.id_attribute_group AND agl.id_lang = '.(int)$id_lang);
        $query->innerJoin('attribute', 'a', 'a.id_attribute_group = agl.id_attribute_group');
        $query->innerJoin('attribute_lang', 'al', 'a.id_attribute = al.id_attribute AND al.id_lang = '.(int)$id_lang);
        $query->innerJoin('product_attribute_combination', 'pac', 'pac.id_attribute = a.id_attribute');
        $query->innerJoin('product_attribute', 'pa', 'pa.id_product_attribute = pac.id_product_attribute');
        if ($ids_product) {
            $query->where('pa.id_product IN ('.implode(',', $ids_product).')');
        }
        $query->groupBy('ag.id_attribute_group, a.id_attribute');

        $result_filter = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query);
        if (is_array($result_filter) && count($result_filter)) {
            $filters_availables = array();

            foreach ($result_filter as $row) {
                //Filtros aplicados
                //----------------------------------------------------
                if ($filters) {
                    foreach ($filters as $i => $id_filter) {
                        $filter = new Attribute((int)$id_filter, $id_lang);
                        if ($filter->id_attribute_group == $row['id_attribute_group'] && $filter->id == $row['id_attribute']) {
                            if (Validate::isLoadedObject($filter)) {
                                $result["filters"]['applied'][] = array(
                                    'param' => 'attribute_'.Tools::link_rewrite($row['public_name']),
                                    'paramLabel' => $row['public_name'],
                                    'value' => (int)$filter->id,
                                    'valueLabel' => $filter->name
                                );

                                unset($filters[$i]);

                                continue 2;
                            }
                        }
                    }
                }
                //----------------------------------------------------

                $filters_availables[Tools::link_rewrite($row['public_name'])]['id_attribute_group'] = $row['id_attribute_group'];
                $filters_availables[Tools::link_rewrite($row['public_name'])]['name'] = $row['public_name'];
                $filters_availables[Tools::link_rewrite($row['public_name'])]['options'][] = array(
                    'value' => (int)$row['id_attribute'],
                    'valueLabel' => $row['name'],
                    'count' => (int)$row['count']
                );
            }

            if ($filters_availables) {
                foreach ($filters_availables as $key => $value) {
                    $filter_available = array();
                    $filter_available['param'] = 'attribute_'.$key;
                    $filter_available['paramLabel'] = $value['name'];
                    $filter_available['options'] = $value['options'];

                    array_push($result["filters"]['available'], $filter_available);
                }
            }
        }
    }

    public function getFiltersCategory(&$result, $ids_product = array())
    {
        $id_lang = Context::getContext()->language->id;
        $filters = trim(strip_tags(Tools::getValue('category')));

        //Filtros disponibles
        //----------------------------------------------------
        $query = new DbQuery();
        $query->select('cp.id_category, cl.name, count(*) as count');
        $query->from('category_product', 'cp');
        $query->innerJoin('category_lang', 'cl', 'cl.id_category = cp.id_category');
        $query->where('cl.id_lang = '.(int)$id_lang);
        if ($ids_product) {
            $query->where('cp.id_product IN ('.implode(',', $ids_product).')');
        }
        if ($filters) {
            $query->where('cp.id_category NOT IN ('.$filters.','.(int)Configuration::get('PS_ROOT_CATEGORY').')');
        } else {
            $query->where('cp.id_category != '.(int)Configuration::get('PS_ROOT_CATEGORY'));
        }
        $query->groupBy('cp.id_category');
        $result_filter = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query);
        if (is_array($result_filter) && count($result_filter)) {
            $filter_available = array();
            $filter_available['param'] = 'category';
            $filter_available['paramLabel'] = $this->module->l('Categories', $this->name_controller);

            foreach ($result_filter as $row) {
                $filter_available['options'][] = array(
                    'value' => (int)$row['id_category'],
                    'valueLabel' => $row['name'],
                    'count' => (int)$row['count']
                );
            }

            array_push($result["filters"]['available'], $filter_available);
        }

        //Filtros aplicados
        //----------------------------------------------------
        $filters = explode(',', $filters);
        if ($filters) {
            foreach ($filters as $id_filter) {
                $filter = new Category((int)$id_filter, $id_lang);
                if (Validate::isLoadedObject($filter)) {
                    $result["filters"]['applied'][] = array(
                        'param' => 'category',
                        'paramLabel' => $this->module->l('Categories', $this->name_controller),
                        'value' => (int)$filter->id,
                        'valueLabel' => $filter->name
                    );
                }
                unset($filter);
            }
        }
    }

    public function getFiltersManufacturer(&$result, $ids_product = array())
    {
        $id_lang = Context::getContext()->language->id;
        $filters = trim(strip_tags(Tools::getValue('manufacturer')));

        //Filtros disponibles
        //----------------------------------------------------
        $query = new DbQuery();
        $query->select('m.id_manufacturer, m.name, count(*) as count');
        $query->from('manufacturer', 'm');
        $query->innerJoin('product', 'p', 'p.id_manufacturer = m.id_manufacturer');
        if ($ids_product) {
            $query->where('p.id_product IN ('.implode(',', $ids_product).')');
        }
        if ($filters) {
            $query->where('m.id_manufacturer NOT IN ('.$filters.')');
        }
        $query->groupBy('m.id_manufacturer');
        $result_filter = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query);

        if (is_array($result_filter) && count($result_filter)) {
            $filter_available = array();
            $filter_available['param'] = 'manufacturer';
            $filter_available['paramLabel'] = $this->module->l('Manufacturers', $this->name_controller);

            foreach ($result_filter as $row) {
                $filter_available['options'][] = array(
                    'value' => (int)$row['id_manufacturer'],
                    'valueLabel' => $row['name'],
                    'count' => (int)$row['count']
                );
            }

            array_push($result["filters"]['available'], $filter_available);
        }

        //Filtros aplicados
        //----------------------------------------------------
        $filters = explode(',', $filters);
        if ($filters) {
            foreach ($filters as $id_filter) {
                $filter = new Manufacturer((int)$id_filter, $id_lang);
                if (Validate::isLoadedObject($filter)) {
                    $result["filters"]['applied'][] = array(
                        'param' => 'manufacturer',
                        'paramLabel' => $this->module->l('Manufacturers', $this->name_controller),
                        'value' => (int)$filter->id,
                        'valueLabel' => $filter->name
                    );
                }
                unset($filter);
            }
        }
    }

    public function getFiltersStock(&$result, $ids_product)
    {
        $id_lang = Context::getContext()->language->id;
        $filters = trim(strip_tags(Tools::getValue('stock')));

        //Filtros disponibles
        //----------------------------------------------------
        $query = new DbQuery();
        $query->select('count(*) as count');
        $query->from('stock_available', 'sa');
        $query->where('sa.quantity > 0');
        $query->where('sa.id_product_attribute = 0');
        if ($ids_product) {
            $query->where('sa.id_product IN ('.implode(',', $ids_product).')');
        }
        $with_stock = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($query);

        $query = new DbQuery();
        $query->select('count(*) as count');
        $query->from('stock_available', 'sa');
        $query->where('sa.quantity <= 0');
        $query->where('sa.id_product_attribute = 0');
        if ($ids_product) {
            $query->where('sa.id_product IN ('.implode(',', $ids_product).')');
        }
        $without_stock = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($query);

        $filter_available = array();
        $filter_available['param'] = 'stock';
        $filter_available['paramLabel'] = $this->module->l('Stock', $this->name_controller);

        if ((int)$filters === 1) {
            $filter_available['options'][] = array(
                'value' => -1,
                'valueLabel' => $this->module->l('No', $this->name_controller),
                'count' => (int)$without_stock
            );
        } else if (Tools::isSubmit('stock')) {
            $filter_available['options'][] = array(
                'value' => 1,
                'valueLabel' => $this->module->l('Yes', $this->name_controller),
                'count' => (int)$with_stock
            );
        } else {
            $filter_available['options'][] = array(
                'value' => -1,
                'valueLabel' => $this->module->l('No', $this->name_controller),
                'count' => (int)$without_stock
            );
            $filter_available['options'][] = array(
                'value' => 1,
                'valueLabel' => $this->module->l('Yes', $this->name_controller),
                'count' => (int)$with_stock
            );
        }

        array_push($result["filters"]['available'], $filter_available);

        //Filtros aplicados
        //----------------------------------------------------
        if ((int)$filters === 1) {
            $result["filters"]['applied'][] = array(
                'param' => 'stock',
                'paramLabel' => $this->module->l('Stock', $this->name_controller),
                'value' => 1,
                'valueLabel' => $this->module->l('Yes', $this->name_controller)
            );
        } else if (Tools::isSubmit('stock')) {
            $result["filters"]['applied'][] = array(
                'param' => 'stock',
                'paramLabel' => $this->module->l('Stock', $this->name_controller),
                'value' => -1,
                'valueLabel' => $this->module->l('No', $this->name_controller)
            );
        }
    }

    /**
     * Obtiene informacion de productos relacionados con el indicado
     * @return array
     */
    public function productRelatedOctMeth()
    {

        $this->setContextLanguage();
        $this->setContextCurrency();
        //$this->setContextCustomer();
        $this->setContextCountry();

        $result = array();
        $result["total"] = 0;
        $result["results"] = array();

        $productId = trim(strip_tags(Tools::getValue('productId')));

        if (isset($productId)) {

            $product = new Product($productId, false, Context::getContext()->language->id);

            //si no se ha cargado devuelvo vacio
            if (!Validate::isLoadedObject($product)) {
                return $result;
            }

            $category = $product->id_category_default;

            //cargo el objeto categorias
            $category = new Category($category, Context::getContext()->language->id);

            //si esta cargado
            if (Validate::isLoadedObject($category)) {

                //compruebo el total,
                $total = $category->getProducts(Context::getContext()->language->id, 1, 100, "position", "asc", true, true);

                if ($total > 1) {

                    $result["total"] = $total - 1;
                    $products = $category->getProducts(Context::getContext()->language->id, 1, 100, "position", "asc", false, true);
                    $products = array_filter($products, function ($v) use ($productId) {
                                //$this->logoutContextCustomer();

                        return $v['id_product'] != $productId;
                    });

                    foreach ($products as $item) {

                        array_push($result["results"], $this->getProductSummary($item['id_product']));

                    }
                    //$this->logoutContextCustomer();

                    return $result;


                } else {
                    //$this->logoutContextCustomer();

                    return $result;
                }

            } else {
                //$this->logoutContextCustomer();

                return $result;
            }


        } else {
            //$this->logoutContextCustomer();

            return $result;
        }
    }

    /**
     * @return array
     * Obtiene informacion sobre el usuario
     */
    public function customerDataOctMeth()
    {
        $this->setContextLanguage();
        $this->setContextCurrency();
        //$this->setContextCustomer();
        $this->setContextCountry();

        $result = array();
        $result["id"] = "";
        $result["firstName"] = "";
        $result["lastName"] = "";
        $result["email"] = "";
        $result["wishlist"] = array();
        $result["cart"] = array();


        $customer = Context::getContext()->customer;

        if (!empty($customer) && Validate::isLoadedObject($customer) && $customer->isLogged() && !$customer->deleted) {

            $result["id"] = $customer->id;
            $result["firstName"] = $customer->firstname;
            $result["lastName"] = $customer->lastname;
            $result["email"] = $customer->email;


            //si esta el modulo de lista blanca activo
            if (Module::isEnabled("blockwishlist") && class_exists("WishList")) {

                Module::getInstanceByName("blockwishlist");

                //obtiene toda la listas de deseis que tiene el usuario
                $wishlist = WishList::getByIdCustomer($customer->id);


                //si tiene una lista de deseos en cookies
                if (isset($this->context->cookie->id_wishlist) && $this->context->cookie->id_wishlist !== '' && !empty($wishlist)) {

                    //cojo solo la de la cookie que sera la ultima
                    $wishlist = array_filter($wishlist, function ($v) {
                        return $v['id_wishlist'] == Context::getContext()->cookie->id_wishlist;
                    });


                    if (!empty($wishlist)) {

                        //obtengo el primer elemento
                        $wishlist = array_shift($wishlist);

                        //recojo los productos
                        $products = WishList::getProductByIdCustomer((int)$wishlist['id_wishlist'], $customer->id, $this->context->language->id, null, true);

                        if (!empty($products)) {

                            foreach ($products as $product) {

                                //los meto en el resultado
                                array_push($result["wishlist"], $this->getProductSummary($product['id_product']));

                            }
                        }
                    }

                }

            }

        }

        $cart = Context::getContext()->cart;

        if (!empty($cart)) {

            $in_cart = $cart->getProducts();

            foreach ($in_cart as $in) {

                $aux = $this->getProductSummary($in["id_product"]);
                $aux["qty"] = $in["cart_quantity"];

                array_push($result["cart"], $aux);

            }
        }
        //$this->logoutContextCustomer();

        return $result;
    }


    /**
     * Agregar productos a la lista blanca
     * @return bool
     * @throws PrestaShopException
     */
    public function addToWishListOctMeth()
    {
        $result = false;

        try {
            if (Module::isEnabled("blockwishlist") && class_exists("WishList")) {

                $customer = Context::getContext()->customer;

                if (!empty($customer) && Validate::isLoadedObject($customer) && $customer->isLogged() && !$customer->deleted) {//Comprobamos que el usuario es accesible

                    $ids = trim(strip_tags(Tools::getValue('productIds')));
                    $ids = explode(',', $ids);
                    Module::getInstanceByName("blockwishlist");

                    $context = Context::getContext();
                    if (!isset($context->cookie->id_wishlist) || $context->cookie->id_wishlist == '') {

                        $wishlist = new WishList();
                        $wishlist->id_shop = $context->shop->id;
                        $wishlist->id_shop_group = $context->shop->id_shop_group;
                        $wishlist->default = 1;

                        $mod_wishlist = new BlockWishList();
                        $wishlist->name = $mod_wishlist->default_wishlist_name;
                        $wishlist->id_customer = (int)$context->customer->id;
                        list($us, $s) = explode(' ', microtime());
                        srand($s * $us);
                        $wishlist->token = Tools::strtoupper(Tools::substr(sha1(uniqid(rand(), true) . _COOKIE_KEY_ . $context->customer->id), 0, 16));
                        $wishlist->add();
                        $context->cookie->id_wishlist = (int)$wishlist->id;
                    }


                    foreach ($ids as $id_product) {

                        $id_product_attribute = (int)Product::getDefaultAttribute($id_product);
                        WishList::addProduct($context->cookie->id_wishlist, $context->customer->id, $id_product, $id_product_attribute, 1);

                    }

                    $result = true;
                }

            } else {

                $result = false;
            }
        } catch (Exception $e) {
            $result = false;
            $this->module->logException($e);
        }

        return $result;

    }

    /**
     * Obtiene el carro de compra
     * @return array
     */
    public function getCartOctMeth()
    {
        $result = array();
        $result["price"] = "0";
        $result["finalPrice"] = "0";
        $result["currency"] = "";
        $result["totalItems"] = "0";
        $result["cart"] = array();

        $cart = Context::getContext()->cart;

        if (!empty($cart)) {

            $result["price"] = $cart->getOrderTotal(false, Cart::ONLY_PRODUCTS);
            $result["finalPrice"] = $cart->getOrderTotal(true, Cart::BOTH);

            $currency = new Currency($cart->id_currency);

            if (Validate::isLoadedObject($currency)) {
                $result["currency"] = $currency->iso_code;
            }

            $in_cart = $cart->getProducts();

            $total = 0;
            foreach ($in_cart as $in) {

                $aux = array();
                $aux["internalId"] = $in["id_product"];
                $aux["title"] = $in["name"];
                $aux["qty"] = $in["cart_quantity"];
                $aux["price"] = $in["price_wt"];
                $total += $in["cart_quantity"];

                array_push($result["cart"], $aux);

            }

            $result["totalItems"] = $total;
        }

        return $result;
    }


    /**
     * Obtiene un informe de los productos vendidos a traves de oct8ne entre dos fechas
     * @return array
     */
    public function getSalesReportOctMeth()
    {

        $from = Tools::getValue("from");
        $to = Tools::getValue("to");
        $apiToken = Tools::getValue("apiToken");

        $in_token = Configuration::get("OCT_API_TOKEN");

        if ($in_token == $apiToken && isset($from) && isset($to)) {

            $result = array();

            return $result;
        }

    }


    /**
     * Devuelve informacion sobre el modulo
     * @return array
     */
    public function getAdapterInfoOctMeth()
    {

        $result = array();
        $result["platform"] = "Prestashop";
        $result["adapterName"] = "Oct8ne official adapter for Prestashop";
        $result["adapterVersion"] = $this->module->version;
        $result["developedBy"] = "Oct8ne Inc";
        $result["supportUrl"] = "";
        $result["apiVersion"] = "2.5";
        $result["enabled"] = $this->module->active == 1;

        return $result;

    }

    /**
     * Obtiene todos los pedidos de un cliente por email.
     * Pedidos validos o no, son devueltos.
     * @return array
     */
    public function getOrdersOctMeth()
    {
        $filterByField = Configuration::get("OTC_ORDER_DETAILS_BY");
        if ( !$filterByField ) $filterByField = 'reference';

        $customerEmail = Tools::getValue("customerEmail");
        $apiToken = Tools::getValue("apiToken");

        //Page por defecto 1
        $page = (int)trim(strip_tags(Tools::getValue('page')));
        if (empty($page)) {
            $page = 1;
        }

        //PageSize por defecto 10
        $pageSize = (int)trim(strip_tags(Tools::getValue('pageSize')));
        if (empty($pageSize)) {
            $pageSize = 10;
        }

		$offset=$pageSize*($page-1);

        $this->setContextLanguage();
        $this->setContextCurrency();
        //$this->setContextCustomer();
        $this->setContextCountry();

        $in_token = Configuration::get("OCT_API_TOKEN");

        $result = array();

        if ($in_token == $apiToken && isset($customerEmail)) {
            $id_customer = Customer::customerExists($customerEmail, true);

            if ($id_customer) {
                $sqlOrders = new DbQuery();
                $sqlOrders->select('o.*, osl.`name` AS order_state');
                $sqlOrders->from('orders', 'o');
                $sqlOrders->leftJoin('order_state_lang', 'osl', 'osl.`id_order_state` = o.`current_state` AND osl.`id_lang` = '.(int)Context::getContext()->language->id);
                $sqlOrders->where('o.`id_customer` = '.(int)$id_customer);
                $sqlOrders->where('o.id_shop = '.(int)$this->context->shop->id);
                $sqlOrders->orderBy('o.`id_order` DESC');
                $sqlOrders->orderBy('o.`date_add` DESC');
                $sqlOrders->limit($pageSize, $offset);
                $orders = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sqlOrders);

                if ($orders) {
                    foreach ($orders as $order) {
                        $currency = Currency::getCurrency($order['id_currency']);

                        $result[] = array(
                            'date' => $order['date_add'],
                            'reference' => $order[$filterByField],
                            'total' => Tools::displayPrice($order['total_paid']),
                            'currency' => $currency['iso_code'],
                            'labelState' => $order['order_state'],
                            'deliveryDate' => ''
                        );
                    }
                }
            }
            //$this->logoutContextCustomer();

            return $result;
        }
    }

    /**
     * Obtiene todos los pedidos de un cliente por email.
     * Pedidos validos o no, son devueltos.
     * @return array
     */
    public function getOrderDetailsOctMeth()
    {
        $filterByField = Configuration::get("OTC_ORDER_DETAILS_BY");
        if ( !$filterByField ) $filterByField = 'reference';
        
        $reference = Tools::getValue("reference");
        $apiToken = Tools::getValue("apiToken");

        $this->setContextLanguage();
        $this->setContextCurrency();
        //$this->setContextCustomer();
        $this->setContextCountry();

        $in_token = Configuration::get("OCT_API_TOKEN");
           
        $result = array();

        if ($in_token == $apiToken && isset($reference)) {
            $sqlOrder = new DbQuery();
            $sqlOrder->select('o.*, osl.name AS order_state, IFNULL(c.name, "Carrier not exist") as carrier_name, oc.tracking_number, c.url as tracking_url');
            $sqlOrder->from('orders', 'o');
            $sqlOrder->innerJoin('order_carrier', 'oc', 'oc.id_order = o.id_order');
            $sqlOrder->leftJoin('carrier', 'c', 'c.id_carrier = oc.id_carrier');
            $sqlOrder->leftJoin('order_state_lang', 'osl', 'osl.id_order_state = o.current_state AND osl.id_lang = '.(int)Context::getContext()->language->id);
            
            if(Tools::substr($filterByField ,0,3 ) == "id_" || $filterByField == "id"){
                $sqlOrder->where('o.' .$filterByField. '= '.(int)pSQL($reference));
            }else{
                $sqlOrder->where('o.' .$filterByField. '= "'.pSQL($reference).'"');
            }
            
            $sqlOrder->where('o.id_shop = '.(int)$this->context->shop->id);
            $sqlOrder->orderBy('o.id_order');
            $sqlOrder->orderBy('o.date_add DESC');
            $order = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($sqlOrder);

            if ($order) {
                $currency = Currency::getCurrency($order['id_currency']);
                $products = array();

                $sqlProductsOrder = new DbQuery();
                $sqlProductsOrder->from('order_detail', 'o');
                $sqlProductsOrder->where('id_order = '.(int)$order['id_order']);
                $productsOrder = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sqlProductsOrder);

                if ($productsOrder) {
                    foreach ($productsOrder as $product) {
                        $products[] = array(
                            'quantity' => $product['product_quantity'],
                            'name' => Product::getProductName(
                                $product['product_id'],
                                $product['product_attribute_id'],
                                Context::getContext()->language->id
                            )
                        );
                    }
                }

                $trackingNumber = $order['tracking_number'];
                $trackingURL = '';
                if (!empty($trackingNumber)) {
                    $trackingURL = Tools::str_replace_once('@', $trackingNumber, $order['tracking_url']);
                }

                $result = array(
                    'date' => $order['date_add'],
                    'reference' => $order[$filterByField],
                    'total' => Tools::displayPrice($order['total_paid']),
                    'currency' => $currency['iso_code'],
                    'labelState' => $order['order_state'],
                    'deliveryDate' => '',
                    'carrier' => $order['carrier_name'],
                    'trackingNumber' => $trackingNumber,
                    'trackingURL' => $trackingURL,
                    'products' => $products
                );

                $messages = CustomerMessage::getMessagesByOrderId((int)$order['id_order']);
                if ($messages) {
                    foreach ($messages as $i => $message) {
                        $result['comments'][$i]['message'] = $message['message'];
                        $result['comments'][$i]['customer'] = empty($message['id_employee']);
                    }
                }
            }
            //$this->logoutContextCustomer();

            return $result;
        }
    }

    /**
     * Obtiene todas las categorias activas del catalogo
     * @return array
     */
    public function getCategoriesOctMeth()
    {
        $this->setContextLanguage();
        $this->setContextCurrency();
        //$this->setContextCustomer();
        $this->setContextCountry();

        $categories = array();

        $this->getCategoriesRecursive($categories, Configuration::get('PS_HOME_CATEGORY'));
        //$this->logoutContextCustomer();

        return $categories;
    }

    private function getCategoriesRecursive(&$categories, $id_category)
    {
        $sql = new DbQuery();
        $sql->from('category', 'c');
        $sql->innerJoin('category_lang', 'cl', 'c.id_category = cl.id_category AND cl.id_lang = '.(int)Context::getContext()->language->id.' AND cl.id_shop = '.(int)$this->context->shop->id);
        $sql->innerJoin('category_shop', 'cs', 'c.id_category = cs.id_category AND cs.id_shop = '.(int)$this->context->shop->id);
        $sql->where('c.id_parent = '.(int)$id_category);
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        if ($result) {
            foreach ($result as $value) {
                $children = array();
                $this->getCategoriesRecursive($children, $value['id_category']);

                if ($this->include_subcategories && is_array($children) && !empty($children)) {
                    $countProducts = 0;
                    foreach($children as $child) {
                        $countProducts += $child['productCount'];
                    }
                } else {
                    $sql1 = new DbQuery();
                    $sql1->select('COUNT(id_category)');
                    $sql1->from('category_product', 'c');
                    $sql1->where('c.id_category = '.(int)$value['id_category']);
                    $countProducts = (int)Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql1);
                }

                $categories[] = array(
                    'id' => (int)$value['id_category'],
                    'parentId' => (int)$value['id_parent'],
                    'name' => $value['name'],
                    'productCount' => $countProducts,
                    'children' => $children
                );
            }
        }
    }

    private function getIdsCategoriesRecursive(&$categories, $id_category)
    {
        $sql = new DbQuery();
        $sql->from('category', 'c');
        $sql->innerJoin('category_shop', 'cs', 'c.id_category = cs.id_category AND cs.id_shop = '.(int)$this->context->shop->id);
        $sql->where('c.id_parent = '.(int)$id_category);
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        if ($result) {
            foreach ($result as $value) {
                $children = array();

                array_push($categories, (int) $value['id_category']);

                $this->getIdsCategoriesRecursive($categories, (int) $value['id_category']);
            }
        }
    }

    /**
     * Obtiene los productos de la categoria enviada.
     * @return array
     */
    public function getProductsByCategoryOctMeth()
    {
        $id_category = Tools::getValue("idCategory");

        $this->setContextLanguage();
        $this->setContextCurrency();
        //$this->setContextCustomer();
        $this->setContextCountry();

        $products = array();

        $sql = new DbQuery();
        $sql->select('id_product');
        $sql->from('category_product', 'cp');
        $sql->where('cp.id_category = '.(int)$id_category);
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        if ($result) {
            foreach ($result as $value) {
                $aux = $this->getProductSummary((int)$value['id_product']);
                if (!empty($aux)) {
                    $products[] = $aux;
                }
            }
        }
        //$this->logoutContextCustomer();

        return array(
            'total' => count($products),
            'result' => $products
        );
    }
}
