<?php

/**
 * Prestashop module for Oct8ne
 *
 * @category  Prestashop
 * @category  Module
 * @author    Prestaquality.com
 * @copyright 2016 Prestaquality
 * @license   Commercial license see license.txt
 * Support by mail  : info@prestaquality.com
 */

require_once dirname(__FILE__) . '/AbstractSearchEngine.php';

/**
 * Implementacion especifica de doofinder
 * Class OctDoofinderSearch
 */
class OctDoofinderSearch extends AbstractSearchEngine
{

    function doSearch($language, $search, $page, $pageSize, $orderBy, $dir)
    {
        //Cargar doofinder.
        $hashId = Configuration::get(Oct8ne::$DOOFINDER_HASH_ID);
        $apiKey = Configuration::get(Oct8ne::$DOOFINDER_API_KEY);

        if (!isset($apiKey) && !isset($hashId)) {
            $doofinder = Module::getInstanceByName("doofinder");
            $aux = $doofinder->searchOnApi($search, $page, $pageSize);
            return $aux;
        } else {
            return $this->executeDoofinderSearchQuery($search, null, $orderBy, $dir, $page, $pageSize);
        }
    }

    function executeDoofinderSearchQuery($searchTerm, $searchCategories, $searchOrder, $searchDir, $page, $pageSize, $includeFacets = TRUE, $method = NULL)
    {
        $hashId = Configuration::get(Oct8ne::$DOOFINDER_HASH_ID);
        $apiKey = Configuration::get(Oct8ne::$DOOFINDER_API_KEY);
        $searchZone = Configuration::get(Oct8ne::$DOOFINDER_SEARCH_ZONE);
        $query_name = Configuration::get(Oct8ne::$DOOFINDER_QUERY_NAME);


        $indices = Configuration::get(Oct8ne::$DOOFINDER_INDICES);
        $idPropertyConf = Configuration::get(Oct8ne::$DOOFINDER_PRODUCTID_PROPERTY);

        if (!isset($idPropertyConf) || $idPropertyConf == "") {
            $idPropertyConf = "id";
        }

        if (!isset($searchZone)) {
            $searchZone = "eu1";
        }

        if(!isset($query_name)){
            $query_name = "match_and";
        }

        $url = "https://##zone##-search.doofinder.com/6/" . $hashId . "/_search";
        $host = str_replace("##zone##", $searchZone, $url);

        $sortLabel = "_score";
        if ($sortLabel != "_score" || ($sortLabel == "_score" && $searchDir != "desc")) {
            $queryParams = array(
                'rpp' => $pageSize, //response results will be formatted as native JSON objects
                'page' => $page,
                'query' => $searchTerm,
                'sort[' . $sortLabel . ']' => $searchDir,
            );
        } else {
            $queryParams = array(
                'rpp' => $pageSize, //response results will be formatted as native JSON objects
                'page' => $page,
                'query' => $searchTerm,
            );
        }

        if ($query_name != "none") {
            $queryParams['query_name'] = $query_name;
        }


        if (isset($indices) && !is_null($indices) && $indices != "") {
            $pos = 0;
            $multipleIndices = explode(",", $indices);
            foreach ($multipleIndices as $index) {
                $queryParams['indices[' . $pos . ']'] = $index;
                $pos = $pos + 1;
            }
        }

        $query = $host . '?' . http_build_query($queryParams);
        $request = curl_init($query);

        curl_setopt($request, CURLOPT_CUSTOMREQUEST, "GET");
        //curl_setopt($request, CURLOPT_POST, 1);
        curl_setopt($request, CURLOPT_RETURNTRANSFER, 1);

        $headers = array(
            'Authorization: Token ' . $apiKey,
            'Content-Type: application/json',
            'Expect:', // Fixes the HTTP/1.1 417 Expectation Failed error
        );

        curl_setopt($request, CURLOPT_HTTPHEADER, $headers);

        $response = curl_exec($request);
        $err = curl_error($request);
        $result = null;
        if (is_null($err) || $err == "") {
            $result = json_decode($response, true);
            if (is_null($result)) {
                curl_close($request);
                return array(
                    'total' => 0,
                    'result' =>  array()
                );
            }
        } else {
            curl_close($request);
            return array(
                'total' => 0,
                'result' =>  array()
            );
        }
        curl_close($request);

        if (!is_null($result)) {
            $products = array();
            foreach ($result["results"] as $item) {
                //$idProperty contains ","
                $idProperty = explode(",", $idPropertyConf);
                foreach ($idProperty as $key => $value) {
                    $value = trim($value);
                    if (isset($item[$value]) && $item[$value] != "") {
                        $product = array(
                            'id_product' => $item[$value]
                        );
                        $products[] = $product;
                        break;
                    }
                }
            }

            return array(
                'total' => $result["total"],
                'result' =>  $products
            );
        }
        return array(
            'total' => 0,
            'result' =>  array()
        );
    }
}
