<?php
/**
 * We offer the best and most useful modules PrestaShop and modifications for your online store.
 *
 * We are experts and professionals in PrestaShop
 *
 * Prestashop module for Oct8ne
 *
 * @author    PresTeamShop.com <support@presteamshop.com>
 * @copyright 2011-2019 PresTeamShop
 * @license   Commercial license see license.txt
 * @category  PrestaShop
 * @category  Module
 */

require_once dirname(__FILE__) . '/AbstractSearchEngine.php';

/**
 * Implementacion especifica de empathybroker
 * Class OctEmpathybrokerSearch
 */
class OctEmpathybrokerSearch extends AbstractSearchEngine
{
    function doSearch($language, $search, $page, $pageSize, $orderBy, $dir)
    {
        $iso_code = Language::getIsoById($language);

        $curl = curl_init();
        $endpoint = Configuration::get(Oct8ne::$URL_API_EMPATHYBROKER);
        $endpointX = strpos($endpoint, "searchX?") !== false;
        
        $search = urlencode($search);
        // Get url
        $url = $endpoint.$search.'&lang='.$iso_code;
        
        // Get start and row parameter. Set them if there are not found, otherwise let it be
        $parts = parse_url($url);
        parse_str($parts['query'], $query);

        if ( !isset( $query['start'] )) {
            $url = $url.'&start='.($page - 1);            
        }
        if ( !isset( $query['rows'] ) ) $url = $url.'&rows='.$pageSize;

        // Execute curl
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);

        $output = curl_exec($curl);

        // Check the return value of curl_exec()
        if ($output === false) {
            throw new Exception(curl_error($curl), curl_errno($curl));
        }
        curl_close($curl);

        $result = Tools::jsonDecode($output);

        $total = (int)$result->content->numFound;
        $products = array();
        if (is_array($result->content->docs) && count($result->content->docs) > 0) {
            foreach ($result->content->docs as $item) {
                if(!$endpointX){
                    $query = new DbQuery();
                    $query->select('p.id_product');
                    $query->from('product', 'p');
                    $query->where('p.reference = \''.pSQL($item->id).'\'');

                    $id_product = (int)Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($query);

                    if (!empty($id_product)) {
                        $product = array(
                            'id_product' => $id_product
                        );

                        $products[] = $product;
                    }
                }else{
                    if (!empty($item->id)) {
                        $product = array(
                            'id_product' => $item->id
                        );

                        $products[] = $product;
                    }
                }
            }
        }

        return array(
            'total' => $total,
            'result' => $products
        );
    }
}